/*
 * Copyright (C) 2008 Emweb bvba, Kessel-Lo, Belgium.
 *
 * See the LICENSE file for terms of use.
 */

#include "ChartsExample2.h"


#include <math.h>
#include <fstream>

#include "CsvUtil.h"

#include <Wt/Chart/WAxisSliderWidget>
#include <Wt/WApplication>
#include <Wt/WDate>
#include <Wt/WEnvironment>
#include <Wt/WItemDelegate>
#include <Wt/WStandardItemModel>
#include <Wt/WText>
#include <Wt/WHBoxLayout>
#include <Wt/WTimer>

#include <Wt/WBorderLayout>
#include <Wt/WFitLayout>

#include <Wt/WStandardItem>
#include <Wt/WTableView>
#include <Wt/WPushButton>
#include <Wt/Chart/WCartesianChart>
#include <Wt/Chart/WPieChart>
#include <Wt/WTimer>
#include <Wt/WDialog>

using namespace Wt;
using namespace Wt::Chart;



namespace {
    

    
  /*
   * A standard item which converts text edits to numbers
   */
  class NumericItem : public WStandardItem {
  public:
    virtual NumericItem *clone() const {
      return new NumericItem();
    }

    virtual void setData(const boost::any &data, int role = UserRole) {
      boost::any dt;

      if (role == EditRole) {
	std::string s = Wt::asString(data).toUTF8();
	char *endptr;
	double d = strtod(s.c_str(), &endptr);
	if (*endptr == 0)
	  dt = boost::any(d);
	else
	  dt = data;
      }

      WStandardItem::setData(data, role);
    }
  };

}


ChartsExample::ChartsExample(WContainerWidget *root)
  : WContainerWidget(root)
{
  new WText(WString::tr("introduction"), this);
  new ScatterPlotExample(this);

}

void  ScatterPlotExample::layoutSizeChanged (int width, int height) 
{
    new WText(WString::tr("layoutSizeChanged!!"), this);
}

ScatterPlotExample::ScatterPlotExample(WContainerWidget *parent):
  WContainerWidget(parent)
{

  this->setLayoutSizeAware(true);

  WStandardItemModel *model = new WStandardItemModel(7000, 4, this);
  model->setItemPrototype(new NumericItem());
  model->setHeaderData(0, WString("A"));
  model->setHeaderData(1, WString("B")); 
  model->setHeaderData(2, WString("C"));

  const auto changeData = [model](double shiftup){
  
    for (unsigned i = 0; i < 700; ++i) {
        double x = (static_cast<double>(i) - 20) / 4;

        model->setData(i, 0, x);
        
        
        if(i <100)
        {
        model->setData(i, 1, -17 + shiftup);
        }
        else if(i>600)
        {
        model->setData(i, 1,  -17 + shiftup);
        }
        else
        {
        model->setData(i, 1,  -17 + sin(x*10)-0.2 + shiftup);
        }
        
        model->setData(i, 3, static_cast<double>(i)/40.0);
        
        model->setData(i, 2, cos(x)*20 + shiftup);
    }
  };
  
  changeData(10);
  /*
   * Create the scatter plot.
   */
  WCartesianChart *chart = new WCartesianChart(this);
  
  chart->setModel(model);        // set the model
  chart->setXSeriesColumn(0);    // set the column that holds the X data
  //chart->setLegendEneabled(true); // enable the legend

  chart->setType(ScatterPlot);   // set type to ScatterPlot

  // Typically, for mathematical functions, you want the axes to cross
  // at the 0 mark:
  chart->axis(XAxis).setLocation(BothSides);
  chart->axis(YAxis).setLocation(BothSides);

  // Automatically layout chart (space for axes, legend, ...)
  //chart->setAutoLayoutEnabled();

  // Add the curves
  WDataSeries s(1, CurveSeries);
  s.setPen(Wt::WPen(Wt::green));
  chart->addSeries(s);
  
  WDataSeries s2(2, CurveSeries);
  chart->addSeries(s2);
  
    
  WDataSeries s3(3, CurveSeries);
  chart->addSeries(s3);

  chart->resize(800, 300); // WPaintedWidget must be given explicit size

  chart->setMargin(10, Top | Bottom);            // add margin vertically
  chart->setMargin(10, Left | Right); // center horizontally
  chart->axis(Wt::Chart::YAxis).setMargin(10);
  chart->axis(Wt::Chart::XAxis).setMargin(10);
  chart->setZoomEnabled(true);
  chart->setPanEnabled(true);
  
  auto curve = Wt::Chart::CurveLabel(*chart->series()[0],WPointF(1.5,sin(1.5)), "test2");
  chart->setCurveManipulationEnabled(true);
  chart->setSeriesSelectionEnabled(true);
  
  chart->addCurveLabel(std::move(curve));
  
    // Add a WAxisSliderWidget for the chart using the data series for column 2

  //Wt::Chart::WAxisSliderWidget *sliderWidget = new Wt::Chart::WAxisSliderWidget(chart->series()[0], parent);

  auto container = new Wt::WContainerWidget(parent);
  auto hbox =  new Wt::WHBoxLayout(container);
  container->setMargin(59);
  Wt::Chart::WAxisSliderWidget *sliderWidget = new Wt::Chart::WAxisSliderWidget();
  hbox->addWidget(sliderWidget);
  sliderWidget->resize(800, 120);
  sliderWidget->setSelectionAreaPadding(40, Wt::Left | Wt::Right);
  sliderWidget->setMargin(Wt::WLength::Auto, Wt::Left | Wt::Right); // Center horizontally
  sliderWidget->setSeries(chart->series()[1]);
  sliderWidget->setSeriesPen(chart->series()[1]->pen());
  sliderWidget->setYAxisZoomEnabled(true);

    
  
  auto timerC = new WTimer(parent);
  timerC->setInterval(200);
  
  timerC->timeout().connect(std::bind([chart, changeData_ = std::move(changeData), counter = 1]() mutable{
      
      changeData_(counter++);
  }));

  timerC->start();

  auto stly = parent->decorationStyle();
  stly.setBackgroundColor(Wt::gray);
  parent->setDecorationStyle(stly);
  

}
